Package: recipes
Title: Preprocessing and Feature Engineering Steps for Modeling
Version: 1.4.0
Authors@R: c(
    person("Max", "Kuhn", , "max@posit.co", role = c("aut", "cre")),
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Emil", "Hvitfeldt", , "emil.hvitfeldt@posit.co", role = "aut"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: A recipe prepares your data for modeling. We provide an
    extensible framework for pipeable sequences of feature engineering
    steps provides preprocessing tools to be applied to data. Statistical
    parameters for the steps can be estimated from an initial data set and
    then applied to other data sets. The resulting processed output can
    then be used as inputs for statistical or machine learning models.
License: MIT + file LICENSE
URL: https://github.com/tidymodels/recipes,
        https://recipes.tidymodels.org/
BugReports: https://github.com/tidymodels/recipes/issues
Depends: dplyr (>= 1.1.0), R (>= 4.1)
Imports: cli, clock (>= 0.6.1), generics (>= 0.1.2), glue, gower,
        hardhat (>= 1.4.1), ipred (>= 0.9-12), lifecycle (>= 1.0.3),
        lubridate (>= 1.8.0), magrittr, Matrix, purrr (>= 1.0.0), rlang
        (>= 1.1.0), sparsevctrs (>= 0.3.3), stats, tibble, tidyr (>=
        1.0.0), tidyselect (>= 1.2.0), timeDate, utils, vctrs (>=
        0.5.0), withr
Suggests: covr, ddalpha, dials (>= 1.2.0), dimRed, fastICA, ggplot2,
        igraph (>= 2.1.1), kernlab, knitr, methods, mixOmics, modeldata
        (>= 0.1.1), parsnip (>= 1.2.0), quarto, RANN, RcppRoll,
        rmarkdown, rpart, rsample, RSpectra, splines2, survival,
        testthat (>= 3.3.0), workflows, xml2
VignetteBuilder: quarto
RdMacros: lifecycle
Config/Needs/website: tidyverse/tidytemplate, rmarkdown
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-23
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-23 20:45:28 UTC; max
Author: Max Kuhn [aut, cre],
  Hadley Wickham [aut],
  Emil Hvitfeldt [aut],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Max Kuhn <max@posit.co>
Repository: CRAN
Date/Publication: 2026-08-24 08:20:02 UTC
Built: R 4.5.2; ; 2026-08-24 09:52:35 UTC; unix
