# Generated by roxygen2: do not edit by hand

S3method(.get_data_types,Date)
S3method(.get_data_types,POSIXct)
S3method(.get_data_types,Surv)
S3method(.get_data_types,character)
S3method(.get_data_types,default)
S3method(.get_data_types,double)
S3method(.get_data_types,factor)
S3method(.get_data_types,hardhat_case_weights)
S3method(.get_data_types,integer)
S3method(.get_data_types,list)
S3method(.get_data_types,logical)
S3method(.get_data_types,numeric)
S3method(.get_data_types,ordered)
S3method(.get_data_types,textrecipes_tokenlist)
S3method(.recipes_estimate_sparsity,default)
S3method(.recipes_estimate_sparsity,recipe)
S3method(.recipes_estimate_sparsity,step_count)
S3method(.recipes_estimate_sparsity,step_dummy)
S3method(.recipes_estimate_sparsity,step_dummy_extract)
S3method(.recipes_estimate_sparsity,step_dummy_multi_choice)
S3method(.recipes_estimate_sparsity,step_holiday)
S3method(.recipes_estimate_sparsity,step_indicate_na)
S3method(.recipes_estimate_sparsity,step_regex)
S3method(.recipes_preserve_sparsity,default)
S3method(.recipes_preserve_sparsity,step_arrange)
S3method(.recipes_preserve_sparsity,step_filter)
S3method(.recipes_preserve_sparsity,step_filter_missing)
S3method(.recipes_preserve_sparsity,step_impute_mean)
S3method(.recipes_preserve_sparsity,step_impute_median)
S3method(.recipes_preserve_sparsity,step_lag)
S3method(.recipes_preserve_sparsity,step_naomit)
S3method(.recipes_preserve_sparsity,step_rename)
S3method(.recipes_preserve_sparsity,step_rename_at)
S3method(.recipes_preserve_sparsity,step_rm)
S3method(.recipes_preserve_sparsity,step_sample)
S3method(.recipes_preserve_sparsity,step_scale)
S3method(.recipes_preserve_sparsity,step_select)
S3method(.recipes_preserve_sparsity,step_shuffle)
S3method(.recipes_preserve_sparsity,step_slice)
S3method(.recipes_preserve_sparsity,step_sqrt)
S3method(.recipes_preserve_sparsity,step_zv)
S3method(bake,check_class)
S3method(bake,check_cols)
S3method(bake,check_missing)
S3method(bake,check_new_values)
S3method(bake,check_range)
S3method(bake,recipe)
S3method(bake,step_BoxCox)
S3method(bake,step_YeoJohnson)
S3method(bake,step_arrange)
S3method(bake,step_bagimpute)
S3method(bake,step_bin2factor)
S3method(bake,step_bs)
S3method(bake,step_center)
S3method(bake,step_classdist)
S3method(bake,step_classdist_shrunken)
S3method(bake,step_corr)
S3method(bake,step_count)
S3method(bake,step_cut)
S3method(bake,step_date)
S3method(bake,step_depth)
S3method(bake,step_discretize)
S3method(bake,step_dummy)
S3method(bake,step_dummy_extract)
S3method(bake,step_dummy_multi_choice)
S3method(bake,step_factor2string)
S3method(bake,step_filter)
S3method(bake,step_filter_missing)
S3method(bake,step_geodist)
S3method(bake,step_harmonic)
S3method(bake,step_holiday)
S3method(bake,step_hyperbolic)
S3method(bake,step_ica)
S3method(bake,step_impute_bag)
S3method(bake,step_impute_knn)
S3method(bake,step_impute_linear)
S3method(bake,step_impute_lower)
S3method(bake,step_impute_mean)
S3method(bake,step_impute_median)
S3method(bake,step_impute_mode)
S3method(bake,step_impute_roll)
S3method(bake,step_indicate_na)
S3method(bake,step_integer)
S3method(bake,step_interact)
S3method(bake,step_intercept)
S3method(bake,step_inverse)
S3method(bake,step_invlogit)
S3method(bake,step_isomap)
S3method(bake,step_knnimpute)
S3method(bake,step_kpca)
S3method(bake,step_kpca_poly)
S3method(bake,step_kpca_rbf)
S3method(bake,step_lag)
S3method(bake,step_lincomb)
S3method(bake,step_log)
S3method(bake,step_logit)
S3method(bake,step_lowerimpute)
S3method(bake,step_meanimpute)
S3method(bake,step_medianimpute)
S3method(bake,step_modeimpute)
S3method(bake,step_mutate)
S3method(bake,step_mutate_at)
S3method(bake,step_naomit)
S3method(bake,step_nnmf)
S3method(bake,step_nnmf_sparse)
S3method(bake,step_normalize)
S3method(bake,step_novel)
S3method(bake,step_ns)
S3method(bake,step_num2factor)
S3method(bake,step_nzv)
S3method(bake,step_ordinalscore)
S3method(bake,step_other)
S3method(bake,step_pca)
S3method(bake,step_percentile)
S3method(bake,step_pls)
S3method(bake,step_poly)
S3method(bake,step_poly_bernstein)
S3method(bake,step_profile)
S3method(bake,step_range)
S3method(bake,step_ratio)
S3method(bake,step_regex)
S3method(bake,step_relevel)
S3method(bake,step_relu)
S3method(bake,step_rename)
S3method(bake,step_rename_at)
S3method(bake,step_rm)
S3method(bake,step_rollimpute)
S3method(bake,step_sample)
S3method(bake,step_scale)
S3method(bake,step_select)
S3method(bake,step_shuffle)
S3method(bake,step_slice)
S3method(bake,step_spatialsign)
S3method(bake,step_spline_b)
S3method(bake,step_spline_convex)
S3method(bake,step_spline_monotone)
S3method(bake,step_spline_natural)
S3method(bake,step_spline_nonnegative)
S3method(bake,step_sqrt)
S3method(bake,step_string2factor)
S3method(bake,step_testthat_helper)
S3method(bake,step_time)
S3method(bake,step_unknown)
S3method(bake,step_unorder)
S3method(bake,step_window)
S3method(bake,step_zv)
S3method(conditionMessage,recipes_error)
S3method(discretize,default)
S3method(discretize,numeric)
S3method(extract_fit_time,recipe)
S3method(extract_parameter_dials,recipe)
S3method(extract_parameter_set_dials,recipe)
S3method(fixed,Date)
S3method(fixed,POSIXct)
S3method(fixed,character)
S3method(fixed,default)
S3method(fixed,factor)
S3method(fixed,numeric)
S3method(formula,recipe)
S3method(predict,discretize)
S3method(prep,check_class)
S3method(prep,check_cols)
S3method(prep,check_missing)
S3method(prep,check_new_values)
S3method(prep,check_range)
S3method(prep,recipe)
S3method(prep,step_BoxCox)
S3method(prep,step_YeoJohnson)
S3method(prep,step_arrange)
S3method(prep,step_bagimpute)
S3method(prep,step_bin2factor)
S3method(prep,step_bs)
S3method(prep,step_center)
S3method(prep,step_classdist)
S3method(prep,step_classdist_shrunken)
S3method(prep,step_corr)
S3method(prep,step_count)
S3method(prep,step_cut)
S3method(prep,step_date)
S3method(prep,step_depth)
S3method(prep,step_discretize)
S3method(prep,step_dummy)
S3method(prep,step_dummy_extract)
S3method(prep,step_dummy_multi_choice)
S3method(prep,step_factor2string)
S3method(prep,step_filter)
S3method(prep,step_filter_missing)
S3method(prep,step_geodist)
S3method(prep,step_harmonic)
S3method(prep,step_holiday)
S3method(prep,step_hyperbolic)
S3method(prep,step_ica)
S3method(prep,step_impute_bag)
S3method(prep,step_impute_knn)
S3method(prep,step_impute_linear)
S3method(prep,step_impute_lower)
S3method(prep,step_impute_mean)
S3method(prep,step_impute_median)
S3method(prep,step_impute_mode)
S3method(prep,step_impute_roll)
S3method(prep,step_indicate_na)
S3method(prep,step_integer)
S3method(prep,step_interact)
S3method(prep,step_intercept)
S3method(prep,step_inverse)
S3method(prep,step_invlogit)
S3method(prep,step_isomap)
S3method(prep,step_knnimpute)
S3method(prep,step_kpca)
S3method(prep,step_kpca_poly)
S3method(prep,step_kpca_rbf)
S3method(prep,step_lag)
S3method(prep,step_lincomb)
S3method(prep,step_log)
S3method(prep,step_logit)
S3method(prep,step_lowerimpute)
S3method(prep,step_meanimpute)
S3method(prep,step_medianimpute)
S3method(prep,step_modeimpute)
S3method(prep,step_mutate)
S3method(prep,step_mutate_at)
S3method(prep,step_naomit)
S3method(prep,step_nnmf)
S3method(prep,step_nnmf_sparse)
S3method(prep,step_normalize)
S3method(prep,step_novel)
S3method(prep,step_ns)
S3method(prep,step_num2factor)
S3method(prep,step_nzv)
S3method(prep,step_ordinalscore)
S3method(prep,step_other)
S3method(prep,step_pca)
S3method(prep,step_percentile)
S3method(prep,step_pls)
S3method(prep,step_poly)
S3method(prep,step_poly_bernstein)
S3method(prep,step_profile)
S3method(prep,step_range)
S3method(prep,step_ratio)
S3method(prep,step_regex)
S3method(prep,step_relevel)
S3method(prep,step_relu)
S3method(prep,step_rename)
S3method(prep,step_rename_at)
S3method(prep,step_rm)
S3method(prep,step_rollimpute)
S3method(prep,step_sample)
S3method(prep,step_scale)
S3method(prep,step_select)
S3method(prep,step_shuffle)
S3method(prep,step_slice)
S3method(prep,step_spatialsign)
S3method(prep,step_spline_b)
S3method(prep,step_spline_convex)
S3method(prep,step_spline_monotone)
S3method(prep,step_spline_natural)
S3method(prep,step_spline_nonnegative)
S3method(prep,step_sqrt)
S3method(prep,step_string2factor)
S3method(prep,step_testthat_helper)
S3method(prep,step_time)
S3method(prep,step_unknown)
S3method(prep,step_unorder)
S3method(prep,step_window)
S3method(prep,step_zv)
S3method(print,check_class)
S3method(print,check_cols)
S3method(print,check_missing)
S3method(print,check_new_values)
S3method(print,check_range)
S3method(print,discretize)
S3method(print,recipe)
S3method(print,step_BoxCox)
S3method(print,step_YeoJohnson)
S3method(print,step_arrange)
S3method(print,step_bagimpute)
S3method(print,step_bin2factor)
S3method(print,step_bs)
S3method(print,step_center)
S3method(print,step_classdist)
S3method(print,step_classdist_shrunken)
S3method(print,step_corr)
S3method(print,step_count)
S3method(print,step_cut)
S3method(print,step_date)
S3method(print,step_depth)
S3method(print,step_discretize)
S3method(print,step_dummy)
S3method(print,step_dummy_extract)
S3method(print,step_dummy_multi_choice)
S3method(print,step_factor2string)
S3method(print,step_filter)
S3method(print,step_filter_missing)
S3method(print,step_geodist)
S3method(print,step_harmonic)
S3method(print,step_holiday)
S3method(print,step_hyperbolic)
S3method(print,step_ica)
S3method(print,step_impute_bag)
S3method(print,step_impute_knn)
S3method(print,step_impute_linear)
S3method(print,step_impute_lower)
S3method(print,step_impute_mean)
S3method(print,step_impute_median)
S3method(print,step_impute_mode)
S3method(print,step_impute_roll)
S3method(print,step_indicate_na)
S3method(print,step_integer)
S3method(print,step_interact)
S3method(print,step_intercept)
S3method(print,step_inverse)
S3method(print,step_invlogit)
S3method(print,step_isomap)
S3method(print,step_knnimpute)
S3method(print,step_kpca)
S3method(print,step_kpca_poly)
S3method(print,step_kpca_rbf)
S3method(print,step_lag)
S3method(print,step_lincomb)
S3method(print,step_log)
S3method(print,step_logit)
S3method(print,step_lowerimpute)
S3method(print,step_meanimpute)
S3method(print,step_medianimpute)
S3method(print,step_modeimpute)
S3method(print,step_mutate)
S3method(print,step_mutate_at)
S3method(print,step_naomit)
S3method(print,step_nnmf)
S3method(print,step_nnmf_sparse)
S3method(print,step_normalize)
S3method(print,step_novel)
S3method(print,step_ns)
S3method(print,step_num2factor)
S3method(print,step_nzv)
S3method(print,step_ordinalscore)
S3method(print,step_other)
S3method(print,step_pca)
S3method(print,step_percentile)
S3method(print,step_pls)
S3method(print,step_poly)
S3method(print,step_poly_bernstein)
S3method(print,step_profile)
S3method(print,step_range)
S3method(print,step_ratio)
S3method(print,step_regex)
S3method(print,step_relevel)
S3method(print,step_relu)
S3method(print,step_rename)
S3method(print,step_rename_at)
S3method(print,step_rm)
S3method(print,step_rollimpute)
S3method(print,step_sample)
S3method(print,step_scale)
S3method(print,step_select)
S3method(print,step_shuffle)
S3method(print,step_slice)
S3method(print,step_spatialsign)
S3method(print,step_spline_b)
S3method(print,step_spline_convex)
S3method(print,step_spline_monotone)
S3method(print,step_spline_natural)
S3method(print,step_spline_nonnegative)
S3method(print,step_sqrt)
S3method(print,step_time)
S3method(print,step_unknown)
S3method(print,step_unorder)
S3method(print,step_window)
S3method(print,step_zv)
S3method(prof,Date)
S3method(prof,POSIXct)
S3method(prof,character)
S3method(prof,factor)
S3method(prof,numeric)
S3method(recipe,data.frame)
S3method(recipe,default)
S3method(recipe,formula)
S3method(recipe,matrix)
S3method(required_pkgs,check)
S3method(required_pkgs,recipe)
S3method(required_pkgs,step)
S3method(required_pkgs,step_classdist_shrunken)
S3method(required_pkgs,step_depth)
S3method(required_pkgs,step_ica)
S3method(required_pkgs,step_isomap)
S3method(required_pkgs,step_kpca)
S3method(required_pkgs,step_kpca_poly)
S3method(required_pkgs,step_kpca_rbf)
S3method(required_pkgs,step_mutate)
S3method(required_pkgs,step_nnmf)
S3method(required_pkgs,step_nnmf_sparse)
S3method(required_pkgs,step_pls)
S3method(required_pkgs,step_poly_bernstein)
S3method(required_pkgs,step_spline_b)
S3method(required_pkgs,step_spline_convex)
S3method(required_pkgs,step_spline_monotone)
S3method(required_pkgs,step_spline_natural)
S3method(required_pkgs,step_spline_nonnegative)
S3method(required_pkgs,step_window)
S3method(summary,recipe)
S3method(tidy,check)
S3method(tidy,check_class)
S3method(tidy,check_cols)
S3method(tidy,check_missing)
S3method(tidy,check_new_values)
S3method(tidy,check_range)
S3method(tidy,recipe)
S3method(tidy,step)
S3method(tidy,step_BoxCox)
S3method(tidy,step_YeoJohnson)
S3method(tidy,step_arrange)
S3method(tidy,step_bagimpute)
S3method(tidy,step_bin2factor)
S3method(tidy,step_bs)
S3method(tidy,step_center)
S3method(tidy,step_classdist)
S3method(tidy,step_classdist_shrunken)
S3method(tidy,step_corr)
S3method(tidy,step_count)
S3method(tidy,step_cut)
S3method(tidy,step_date)
S3method(tidy,step_depth)
S3method(tidy,step_discretize)
S3method(tidy,step_dummy)
S3method(tidy,step_dummy_extract)
S3method(tidy,step_dummy_multi_choice)
S3method(tidy,step_factor2string)
S3method(tidy,step_filter)
S3method(tidy,step_filter_missing)
S3method(tidy,step_geodist)
S3method(tidy,step_harmonic)
S3method(tidy,step_holiday)
S3method(tidy,step_hyperbolic)
S3method(tidy,step_ica)
S3method(tidy,step_impute_bag)
S3method(tidy,step_impute_knn)
S3method(tidy,step_impute_linear)
S3method(tidy,step_impute_lower)
S3method(tidy,step_impute_mean)
S3method(tidy,step_impute_median)
S3method(tidy,step_impute_mode)
S3method(tidy,step_impute_roll)
S3method(tidy,step_indicate_na)
S3method(tidy,step_integer)
S3method(tidy,step_interact)
S3method(tidy,step_intercept)
S3method(tidy,step_inverse)
S3method(tidy,step_invlogit)
S3method(tidy,step_isomap)
S3method(tidy,step_knnimpute)
S3method(tidy,step_kpca)
S3method(tidy,step_kpca_poly)
S3method(tidy,step_kpca_rbf)
S3method(tidy,step_lag)
S3method(tidy,step_lincomb)
S3method(tidy,step_log)
S3method(tidy,step_logit)
S3method(tidy,step_lowerimpute)
S3method(tidy,step_meanimpute)
S3method(tidy,step_medianimpute)
S3method(tidy,step_modeimpute)
S3method(tidy,step_mutate)
S3method(tidy,step_mutate_at)
S3method(tidy,step_naomit)
S3method(tidy,step_nnmf)
S3method(tidy,step_nnmf_sparse)
S3method(tidy,step_normalize)
S3method(tidy,step_novel)
S3method(tidy,step_ns)
S3method(tidy,step_num2factor)
S3method(tidy,step_nzv)
S3method(tidy,step_ordinalscore)
S3method(tidy,step_other)
S3method(tidy,step_pca)
S3method(tidy,step_percentile)
S3method(tidy,step_pls)
S3method(tidy,step_poly)
S3method(tidy,step_poly_bernstein)
S3method(tidy,step_profile)
S3method(tidy,step_range)
S3method(tidy,step_ratio)
S3method(tidy,step_regex)
S3method(tidy,step_relevel)
S3method(tidy,step_relu)
S3method(tidy,step_rename)
S3method(tidy,step_rename_at)
S3method(tidy,step_rm)
S3method(tidy,step_rollimpute)
S3method(tidy,step_sample)
S3method(tidy,step_scale)
S3method(tidy,step_select)
S3method(tidy,step_shuffle)
S3method(tidy,step_slice)
S3method(tidy,step_spatialsign)
S3method(tidy,step_spline_b)
S3method(tidy,step_spline_convex)
S3method(tidy,step_spline_monotone)
S3method(tidy,step_spline_natural)
S3method(tidy,step_spline_nonnegative)
S3method(tidy,step_sqrt)
S3method(tidy,step_string2factor)
S3method(tidy,step_time)
S3method(tidy,step_unknown)
S3method(tidy,step_unorder)
S3method(tidy,step_window)
S3method(tidy,step_zv)
S3method(tunable,check)
S3method(tunable,recipe)
S3method(tunable,step)
S3method(tunable,step_bagimpute)
S3method(tunable,step_bs)
S3method(tunable,step_classdist_shrunken)
S3method(tunable,step_corr)
S3method(tunable,step_discretize)
S3method(tunable,step_dummy_multi_choice)
S3method(tunable,step_filter_missing)
S3method(tunable,step_harmonic)
S3method(tunable,step_ica)
S3method(tunable,step_impute_bag)
S3method(tunable,step_impute_knn)
S3method(tunable,step_impute_mean)
S3method(tunable,step_impute_roll)
S3method(tunable,step_isomap)
S3method(tunable,step_knnimpute)
S3method(tunable,step_kpca_poly)
S3method(tunable,step_kpca_rbf)
S3method(tunable,step_meanimpute)
S3method(tunable,step_nnmf)
S3method(tunable,step_nnmf_sparse)
S3method(tunable,step_ns)
S3method(tunable,step_nzv)
S3method(tunable,step_other)
S3method(tunable,step_pca)
S3method(tunable,step_pls)
S3method(tunable,step_poly)
S3method(tunable,step_poly_bernstein)
S3method(tunable,step_rollimpute)
S3method(tunable,step_spline_b)
S3method(tunable,step_spline_convex)
S3method(tunable,step_spline_monotone)
S3method(tunable,step_spline_natural)
S3method(tunable,step_spline_nonnegative)
S3method(tunable,step_window)
S3method(tune_args,check)
S3method(tune_args,recipe)
S3method(tune_args,step)
S3method(update,step)
export("%>%")
export(.get_data_types)
export(.recipes_estimate_sparsity)
export(.recipes_preserve_sparsity)
export(.recipes_toggle_sparse_args)
export(add_check)
export(add_role)
export(add_step)
export(all_date)
export(all_date_predictors)
export(all_datetime)
export(all_datetime_predictors)
export(all_double)
export(all_double_predictors)
export(all_factor)
export(all_factor_predictors)
export(all_integer)
export(all_integer_predictors)
export(all_logical)
export(all_logical_predictors)
export(all_nominal)
export(all_nominal_predictors)
export(all_numeric)
export(all_numeric_predictors)
export(all_ordered)
export(all_ordered_predictors)
export(all_outcomes)
export(all_predictors)
export(all_string)
export(all_string_predictors)
export(all_unordered)
export(all_unordered_predictors)
export(are_weights_used)
export(averages)
export(bake)
export(check)
export(check_class)
export(check_cols)
export(check_missing)
export(check_name)
export(check_new_data)
export(check_new_values)
export(check_options)
export(check_range)
export(check_type)
export(correlations)
export(covariances)
export(current_info)
export(denom_vars)
export(detect_step)
export(discretize)
export(dummy_extract_names)
export(dummy_names)
export(ellipse_check)
export(estimate_yj)
export(extract_fit_time)
export(extract_parameter_dials)
export(extract_parameter_set_dials)
export(fixed)
export(format_ch_vec)
export(format_selectors)
export(frequency_weights)
export(fully_trained)
export(get_case_weights)
export(get_keep_original_cols)
export(has_role)
export(has_type)
export(imp_vars)
export(importance_weights)
export(is_trained)
export(juice)
export(medians)
export(names0)
export(pca_wts)
export(prep)
export(prepare)
export(prepper)
export(print_step)
export(printer)
export(prof)
export(rand_id)
export(recipe)
export(recipes_argument_select)
export(recipes_eval_select)
export(recipes_extension_check)
export(recipes_map_cols)
export(recipes_names_outcomes)
export(recipes_names_predictors)
export(recipes_pkg_check)
export(recipes_ptype)
export(recipes_ptype_validate)
export(recipes_remove_cols)
export(remove_original_cols)
export(remove_role)
export(required_pkgs)
export(sel2char)
export(step)
export(step_BoxCox)
export(step_YeoJohnson)
export(step_arrange)
export(step_bagimpute)
export(step_bin2factor)
export(step_bs)
export(step_center)
export(step_classdist)
export(step_classdist_shrunken)
export(step_corr)
export(step_count)
export(step_cut)
export(step_date)
export(step_depth)
export(step_discretize)
export(step_dummy)
export(step_dummy_extract)
export(step_dummy_multi_choice)
export(step_factor2string)
export(step_filter)
export(step_filter_missing)
export(step_geodist)
export(step_harmonic)
export(step_holiday)
export(step_hyperbolic)
export(step_ica)
export(step_impute_bag)
export(step_impute_knn)
export(step_impute_linear)
export(step_impute_lower)
export(step_impute_mean)
export(step_impute_median)
export(step_impute_mode)
export(step_impute_roll)
export(step_indicate_na)
export(step_integer)
export(step_interact)
export(step_intercept)
export(step_inverse)
export(step_invlogit)
export(step_isomap)
export(step_knnimpute)
export(step_kpca)
export(step_kpca_poly)
export(step_kpca_rbf)
export(step_lag)
export(step_lincomb)
export(step_log)
export(step_logit)
export(step_lowerimpute)
export(step_meanimpute)
export(step_medianimpute)
export(step_modeimpute)
export(step_mutate)
export(step_mutate_at)
export(step_naomit)
export(step_nnmf)
export(step_nnmf_sparse)
export(step_normalize)
export(step_novel)
export(step_ns)
export(step_num2factor)
export(step_nzv)
export(step_ordinalscore)
export(step_other)
export(step_pca)
export(step_percentile)
export(step_pls)
export(step_poly)
export(step_poly_bernstein)
export(step_profile)
export(step_range)
export(step_ratio)
export(step_regex)
export(step_relevel)
export(step_relu)
export(step_rename)
export(step_rename_at)
export(step_rm)
export(step_rollimpute)
export(step_sample)
export(step_scale)
export(step_select)
export(step_shuffle)
export(step_slice)
export(step_spatialsign)
export(step_spline_b)
export(step_spline_convex)
export(step_spline_monotone)
export(step_spline_natural)
export(step_spline_nonnegative)
export(step_sqrt)
export(step_string2factor)
export(step_time)
export(step_unknown)
export(step_unorder)
export(step_window)
export(step_zv)
export(terms_select)
export(tidy)
export(tunable)
export(tune_args)
export(update)
export(update_role)
export(update_role_requirements)
export(variances)
export(yj_transform)
import(rlang)
import(timeDate)
importFrom(Matrix,Matrix)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,do)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(generics,required_pkgs)
importFrom(generics,tidy)
importFrom(generics,tunable)
importFrom(generics,tune_args)
importFrom(glue,glue)
importFrom(gower,gower_topn)
importFrom(hardhat,extract_fit_time)
importFrom(hardhat,extract_parameter_dials)
importFrom(hardhat,extract_parameter_set_dials)
importFrom(hardhat,frequency_weights)
importFrom(hardhat,importance_weights)
importFrom(ipred,ipredbagg)
importFrom(lifecycle,deprecated)
importFrom(lubridate,am)
importFrom(lubridate,decimal_date)
importFrom(lubridate,hour)
importFrom(lubridate,is.Date)
importFrom(lubridate,mday)
importFrom(lubridate,minute)
importFrom(lubridate,month)
importFrom(lubridate,quarter)
importFrom(lubridate,second)
importFrom(lubridate,semester)
importFrom(lubridate,wday)
importFrom(lubridate,week)
importFrom(lubridate,yday)
importFrom(lubridate,year)
importFrom(magrittr,"%>%")
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_dbl)
importFrom(purrr,map_lgl)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,cov)
importFrom(stats,cov.wt)
importFrom(stats,lm)
importFrom(stats,mahalanobis)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,optimize)
importFrom(stats,poly)
importFrom(stats,prcomp)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(tibble,add_column)
importFrom(tibble,as_tibble)
importFrom(tibble,is_tibble)
importFrom(tibble,tibble)
importFrom(tidyselect,everything)
importFrom(utils,install.packages)
importFrom(utils,object.size)
importFrom(vctrs,vec_cast)
importFrom(vctrs,vec_cbind)
importFrom(vctrs,vec_detect_complete)
importFrom(vctrs,vec_slice)
