
#' @importFrom generics tidy
#' @export
generics::tidy

#' @importFrom generics required_pkgs
#' @export
generics::required_pkgs

#' @importFrom generics tunable
#' @export
generics::tunable

#' @importFrom generics tune_args
#' @export
generics::tune_args

#' @importFrom hardhat extract_parameter_set_dials
#' @export
hardhat::extract_parameter_set_dials

#' @importFrom hardhat extract_parameter_dials
#' @export
hardhat::extract_parameter_dials

#' @importFrom hardhat importance_weights
#' @export
hardhat::importance_weights

#' @importFrom hardhat frequency_weights
#' @export
hardhat::frequency_weights

#' @importFrom magrittr %>%
#' @export
magrittr::`%>%`
