# nocov start

# Global vars ------------------------------------------------------------------

utils::globalVariables(
  c(
    # roles.R
    ".",
    ".orig_order",
    # misc.R
    "type",
    "new_type",
    # novel.R
    "variable",
    # lowerimpute.R
    "estimate",
    # integer.R
    ".row",
    # ratio.R
    "denom",
    # ica.R
    "component",
    "denom",
    "id",
    "value",
    # pls.R
    "training",
    "x_names",
    "y_names",
    # recipes.R
    ".order_1",
    ".order_2",
    "number",
    "role",
    "skip",
    # extract.R
    "call_info",
    # nnmf_sparse.R
    "dat",
    # classdist_shrunken.R
    "..wts",
    "..y",
    "by_class",
    "class_n",
    "delta",
    "delta_wts",
    "distance",
    "global",
    "msq",
    "scaled_value",
    "shrink",
    "shrunken",
    "sq_df",
    "sq_diff",
    "std_dev"
  )
)

# nocov end
