### step_bagimpute -------------------------------------------------------------

#' @keywords internal
#' @export
prep.step_bagimpute <- prep.step_impute_bag

#' @keywords internal
#' @export
bake.step_bagimpute <- bake.step_impute_bag

#' @keywords internal
#' @export
print.step_bagimpute <- print.step_impute_bag

#' @keywords internal
#' @export
tidy.step_bagimpute <- tidy.step_impute_bag

#' @keywords internal
#' @export
tunable.step_bagimpute <- tunable.step_impute_bag

### step_knnimpute -------------------------------------------------------------

#' @keywords internal
#' @export
prep.step_knnimpute <- prep.step_impute_knn

#' @keywords internal
#' @export
bake.step_knnimpute <- bake.step_impute_knn

#' @keywords internal
#' @export
print.step_knnimpute <- print.step_impute_knn

#' @keywords internal
#' @export
tidy.step_knnimpute <- tidy.step_impute_knn

#' @keywords internal
#' @export
tunable.step_knnimpute <- tunable.step_impute_knn

### step_lowerimpute -----------------------------------------------------------

#' @keywords internal
#' @export
prep.step_lowerimpute <- prep.step_impute_lower

#' @keywords internal
#' @export
bake.step_lowerimpute <- bake.step_impute_lower

#' @keywords internal
#' @export
print.step_lowerimpute <- print.step_impute_lower

#' @keywords internal
#' @export
tidy.step_lowerimpute <- tidy.step_impute_lower

### step_meanimpute ------------------------------------------------------------

#' @keywords internal
#' @export
prep.step_meanimpute <- prep.step_impute_mean

#' @keywords internal
#' @export
bake.step_meanimpute <- bake.step_impute_mean

#' @keywords internal
#' @export
print.step_meanimpute <- print.step_impute_mean

#' @keywords internal
#' @export
tidy.step_meanimpute <- tidy.step_impute_mean

#' @keywords internal
#' @export
tunable.step_meanimpute <- tunable.step_impute_mean

### step_medianimpute ----------------------------------------------------------

#' @keywords internal
#' @export
prep.step_medianimpute <- prep.step_impute_median

#' @keywords internal
#' @export
bake.step_medianimpute <- bake.step_impute_median

#' @keywords internal
#' @export
print.step_medianimpute <- print.step_impute_median

#' @keywords internal
#' @export
tidy.step_medianimpute <- tidy.step_impute_median

### step_modeimpute ------------------------------------------------------------

#' @keywords internal
#' @export
prep.step_modeimpute <- prep.step_impute_mode

#' @keywords internal
#' @export
bake.step_modeimpute <- bake.step_impute_mode

#' @keywords internal
#' @export
print.step_modeimpute <- print.step_impute_mode

#' @keywords internal
#' @export
tidy.step_modeimpute <- tidy.step_impute_mode

### step_rollimpute ------------------------------------------------------------

#' @keywords internal
#' @export
prep.step_rollimpute <- prep.step_impute_roll

#' @keywords internal
#' @export
bake.step_rollimpute <- bake.step_impute_roll

#' @keywords internal
#' @export
print.step_rollimpute <- print.step_impute_roll

#' @keywords internal
#' @export
tidy.step_rollimpute <- tidy.step_impute_roll

#' @keywords internal
#' @export
tunable.step_rollimpute <- tunable.step_impute_roll
