---
title: "Subsampling for Class Imbalances"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Subsampling for Class Imbalances}
output:
  knitr:::html_vignette:
    toc: yes
---

This article has been moved to [`tidymodels.org`](https://www.tidymodels.org/learn/models/sub-sampling/). 

