---
title: "Multivariate Analysis using Partial Least Squares"
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Multivariate Analysis using Partial Least Squares}
output:
  knitr:::html_vignette:
    toc: yes
---

This article can now be found at [`tidymodels.org`](https://www.tidymodels.org/learn/models/pls/).
