# Deprecation error

    Code
      step_meanimpute(recipe(~., data = mtcars))
    Condition
      Error:
      ! `step_meanimpute()` was deprecated in recipes 0.1.16 and is now defunct.
      i Please use `step_impute_mean()` instead.

