% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{rand_id}
\alias{rand_id}
\title{Make a random identification field for steps}
\usage{
rand_id(prefix = "step", len = 5)
}
\arguments{
\item{prefix}{A single character string}

\item{len}{An integer for the number of random characters}
}
\value{
A character string with the prefix and random letters separated by
and underscore.
}
\description{
Make a random identification field for steps
}
\keyword{internal}
