% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/impute_lower.R
\name{step_impute_lower}
\alias{step_impute_lower}
\title{Impute numeric data below the threshold of measurement}
\usage{
step_impute_lower(
  recipe,
  ...,
  role = NA,
  trained = FALSE,
  threshold = NULL,
  skip = FALSE,
  id = rand_id("impute_lower")
)
}
\arguments{
\item{recipe}{A recipe object. The step will be added to the sequence of
operations for this recipe.}

\item{...}{One or more selector functions to choose variables for this step.
See \code{\link[=selections]{selections()}} for more details.}

\item{role}{Not used by this step since no new variables are created.}

\item{trained}{A logical to indicate if the quantities for preprocessing have
been estimated.}

\item{threshold}{A named numeric vector of lower bounds. This is \code{NULL} until
computed by \code{\link[=prep]{prep()}}.}

\item{skip}{A logical. Should the step be skipped when the recipe is baked by
\code{\link[=bake]{bake()}}? While all operations are baked when \code{\link[=prep]{prep()}} is run, some
operations may not be able to be conducted on new data (e.g. processing the
outcome variable(s)). Care should be taken when using \code{skip = TRUE} as it
may affect the computations for subsequent operations.}

\item{id}{A character string that is unique to this step to identify it.}
}
\value{
An updated version of \code{recipe} with the new step added to the
sequence of any existing operations.
}
\description{
\code{step_impute_lower()} creates a \emph{specification} of a recipe step designed for
cases where the non-negative numeric data cannot be measured below a known
value. In these cases, one method for imputing the data is to substitute the
truncated value by a random uniform number between zero and the truncation
point.
}
\details{
\code{step_impute_lower()} estimates the variable minimums from the data used in
the \code{training} argument of \code{\link[=prep]{prep()}}. \code{\link[=bake]{bake()}} then simulates a value for any
data at the minimum with a random uniform value between zero and the minimum.

As of \code{recipes} 0.1.16, this function name changed from \code{step_lowerimpute()}
to \code{step_impute_lower()}.
}
\section{Tidying}{
When you \code{\link[=tidy.recipe]{tidy()}} this step, a tibble is returned with
columns \code{terms}, \code{value} , and \code{id}:

\describe{
\item{terms}{character, the selectors or variables selected}
\item{value}{numeric, the estimated value}
\item{id}{character, id of this step}
}
}

\section{Case weights}{


The underlying operation does not allow for case weights.
}

\examples{
\dontshow{if (rlang::is_installed("modeldata")) withAutoprint(\{ # examplesIf}
library(recipes)
data(biomass, package = "modeldata")

## Truncate some values to emulate what a lower limit of
## the measurement system might look like

biomass$carbon <- ifelse(biomass$carbon > 40, biomass$carbon, 40)
biomass$hydrogen <- ifelse(biomass$hydrogen > 5, biomass$carbon, 5)

biomass_tr <- biomass[biomass$dataset == "Training", ]
biomass_te <- biomass[biomass$dataset == "Testing", ]

rec <- recipe(
  HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
  data = biomass_tr
)

impute_rec <- rec |>
  step_impute_lower(carbon, hydrogen)

tidy(impute_rec, number = 1)

impute_rec <- prep(impute_rec, training = biomass_tr)

tidy(impute_rec, number = 1)

transformed_te <- bake(impute_rec, biomass_te)

plot(transformed_te$carbon, biomass_te$carbon,
  ylab = "pre-imputation", xlab = "imputed"
)
\dontshow{\}) # examplesIf}
}
\seealso{
Other imputation steps:
\code{\link[=step_impute_bag]{step_impute_bag()}},
\code{\link[=step_impute_knn]{step_impute_knn()}},
\code{\link[=step_impute_linear]{step_impute_linear()}},
\code{\link[=step_impute_mean]{step_impute_mean()}},
\code{\link[=step_impute_median]{step_impute_median()}},
\code{\link[=step_impute_mode]{step_impute_mode()}},
\code{\link[=step_impute_roll]{step_impute_roll()}}
}
\concept{imputation steps}
