% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ns.R
\name{step_ns}
\alias{step_ns}
\title{Natural spline basis functions}
\usage{
step_ns(
  recipe,
  ...,
  role = "predictor",
  trained = FALSE,
  objects = NULL,
  deg_free = 2,
  options = list(),
  keep_original_cols = FALSE,
  skip = FALSE,
  id = rand_id("ns")
)
}
\arguments{
\item{recipe}{A recipe object. The step will be added to the sequence of
operations for this recipe.}

\item{...}{One or more selector functions to choose variables for this step.
See \code{\link[=selections]{selections()}} for more details.}

\item{role}{For model terms created by this step, what analysis role should
they be assigned? By default, the new columns created by this step from the
original variables will be used as \emph{predictors} in a model.}

\item{trained}{A logical to indicate if the quantities for preprocessing have
been estimated.}

\item{objects}{A list of \code{\link[splines:ns]{splines::ns()}} objects created once the step has
been trained.}

\item{deg_free}{The degrees of freedom for the natural spline. As the degrees
of freedom for a natural spline increase, more flexible and complex curves
can be generated. When a single degree of freedom is used, the result is a
rescaled version of the original data.}

\item{options}{A list of options for \code{\link[splines:ns]{splines::ns()}} which should not include
\code{x} or \code{df}.}

\item{keep_original_cols}{A logical to keep the original variables in the
output. Defaults to \code{FALSE}.}

\item{skip}{A logical. Should the step be skipped when the recipe is baked by
\code{\link[=bake]{bake()}}? While all operations are baked when \code{\link[=prep]{prep()}} is run, some
operations may not be able to be conducted on new data (e.g. processing the
outcome variable(s)). Care should be taken when using \code{skip = TRUE} as it
may affect the computations for subsequent operations.}

\item{id}{A character string that is unique to this step to identify it.}
}
\value{
An updated version of \code{recipe} with the new step added to the
sequence of any existing operations.
}
\description{
\code{step_ns()} creates a \emph{specification} of a recipe step that will create new
columns that are basis expansions of variables using natural splines.
}
\details{
\code{step_ns()} can create new features from a single variable that enable
fitting routines to model this variable in a nonlinear manner. The extent of
the possible nonlinearity is determined by the \code{df} or \code{knots} arguments of
\code{\link[splines:ns]{splines::ns()}}. The original variables are removed from the data and new
columns are added. The naming convention for the new variables is
\code{varname_ns_1} and so on.
}
\section{Tidying}{
When you \code{\link[=tidy.recipe]{tidy()}} this step, a tibble is returned with
columns \code{terms} and \code{id}:

\describe{
\item{terms}{character, the selectors or variables selected}
\item{id}{character, id of this step}
}
}

\section{Tuning Parameters}{
This step has 1 tuning parameter:
\itemize{
\item \code{deg_free}: Spline Degrees of Freedom (type: integer, default: 2)
}
}

\section{Case weights}{


The underlying operation does not allow for case weights.
}

\examples{
\dontshow{if (rlang::is_installed("modeldata")) withAutoprint(\{ # examplesIf}
data(biomass, package = "modeldata")

biomass_tr <- biomass[biomass$dataset == "Training", ]
biomass_te <- biomass[biomass$dataset == "Testing", ]

rec <- recipe(
  HHV ~ carbon + hydrogen + oxygen + nitrogen + sulfur,
  data = biomass_tr
)

with_splines <- rec |>
  step_ns(carbon, hydrogen)
with_splines <- prep(with_splines, training = biomass_tr)

expanded <- bake(with_splines, biomass_te)
expanded
\dontshow{\}) # examplesIf}
}
\seealso{
Other individual transformation steps:
\code{\link[=step_BoxCox]{step_BoxCox()}},
\code{\link[=step_YeoJohnson]{step_YeoJohnson()}},
\code{\link[=step_bs]{step_bs()}},
\code{\link[=step_harmonic]{step_harmonic()}},
\code{\link[=step_hyperbolic]{step_hyperbolic()}},
\code{\link[=step_inverse]{step_inverse()}},
\code{\link[=step_invlogit]{step_invlogit()}},
\code{\link[=step_log]{step_log()}},
\code{\link[=step_logit]{step_logit()}},
\code{\link[=step_mutate]{step_mutate()}},
\code{\link[=step_percentile]{step_percentile()}},
\code{\link[=step_poly]{step_poly()}},
\code{\link[=step_relu]{step_relu()}},
\code{\link[=step_sqrt]{step_sqrt()}}
}
\concept{individual transformation steps}
